;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x7935e10700910b5f052ca4a2d9f7d5f5 . "./code.cg2js/cg-acache.fasl")
(#x803b719feea8ca1d7d37e7cb48dcd543 . "./code.cg2js/cg-base.fasl")
(#x10ab5fb239749f6599153b0079a69c2b . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xeaceec6fcb9341d96d7509d28f1e7ced . "./code.cg2js/cg-bitmap-pane.fasl")
(#xc97df7e32dce16d311d7d2ede19546db . "./code.cg2js/cg-bitmap-stream.fasl")
(#x061e64f4780cedc514d6928a9810d12c . "./code.cg2js/cg-button.fasl")
(#x601856ed64ff151b7662b09e55fb968c . "./code.cg2js/cg-calendar.fasl")
(#x3e290e3b1f3c6999cfdf8bd9f60ac696 . "./code.cg2js/cg-caret.fasl")
(#xea568f3aa2c05bdf872cd24e3bb55a12 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xbb6ed786369142e40bf10f4fbcac3a5c . "./code.cg2js/cg-chart-widget.fasl")
(#xd07440b6727c98984eb0c182b72920a3 . "./code.cg2js/cg-check-box.fasl")
(#x54470730307fd8eceef25ff4c7796feb . "./code.cg2js/cg-choice-list.fasl")
(#x320ed9f398c0737f1aa4f2a4bc64caf0 . "./code.cg2js/cg-choose-printer.fasl")
(#x99659d2e264cc740af33caa39137ffd3 . "./code.cg2js/cg-class-grid.fasl")
(#x9afa3f45186f37134a865e890d6dff40 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x2c19e4e8a0f20f1e458fc3f561120786 . "./code.cg2js/cg-class-support.fasl")
(#x1481c07f54cba44439aae4cc9ab36ffe . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x4f218f12a6f536cccb1c0de7c77da143 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x4ce8a6036b13e90325083b7f98cc0dc4 . "./code.cg2js/cg-clipboard.fasl")
(#xb868bfec0e68f9636c83cea7e821bdc5 . "./code.cg2js/cg-color-dialog.fasl")
(#x271e10d8c6f60c1fe1c9323b357da898 . "./code.cg2js/cg-combo-box.fasl")
(#x6aa93a65ad646d7d2e4fee94aa193dcc . "./code.cg2js/cg-common-control.fasl")
(#xa3c72b92373a604f7540d5080212168e . "./code.cg2js/cg-comtab.fasl")
(#x1c2d7a7651120845868e9623dc8f3e41 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x09702256683de71d48b1af7c7561ad1c . "./code.cg2js/cg-curve.fasl")
(#x84ed59246f59303c2a87f691f3e2e96d . "./code.cg2js/cg-dde-utils.fasl")
(#x5d58f5a2f98126cc50a1ed4193280bfa . "./code.cg2js/cg-dev.fasl")
(#x65f0c0f0cf5d02ff7eea9cfd808e7220 . "./code.cg2js/cg-dialog-item.fasl")
(#x03d2ebb2e5dba6aa14db343c8216b342 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x790f4724f416f100f5ae45c81dae841c . "./code.cg2js/cg-directory-dialog.fasl")
(#xc4df8cb1b172f9a4f169634df964e8df . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x228bbbe891c451dd163f831c86ac37f9 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x6ca25cb5e0cb8254424774cf17907180 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x317a6f2b7d7921ced9830a99eec742d9 . "./code.cg2js/cg-drawable.fasl")
(#x0444b496e44b939021ae2ec9f9b1389c . "./code.cg2js/cg-dropping-outline.fasl")
(#x0a837199e5650a4e3d8c00daaa861106 . "./code.cg2js/cg-edit-in-place.fasl")
(#x3d23b4ab7d82570033bf70191a93879b . "./code.cg2js/cg-editable-text.fasl")
(#xed89ffc125b5188ee2ceb653e8f630a5 . "./code.cg2js/cg-file-dialog.fasl")
(#x81ba735f821753fc788deb10df298c86 . "./code.cg2js/cg-fill-texture.fasl")
(#x907262e384655ee723a8647cef613eac . "./code.cg2js/cg-find-string-dialog.fasl")
(#x6515475868f609786580bd99e31ad22b . "./code.cg2js/cg-font-dialog.fasl")
(#x96e17c4f78cb9d363688e9bfb5096169 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x8818223d0c0ae45cc454e211514ce315 . "./code.cg2js/cg-get-pixmap.fasl")
(#x450dbe955a4fa6043e210db0681dc9fc . "./code.cg2js/cg-get-position.fasl")
(#xc16e9b4cae747654db4016ef805a5acf . "./code.cg2js/cg-graphics-context.fasl")
(#x7dcf93409f855ca44f7329b0a2f955c9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x9a107ff59f67161710ce0d20dfbad515 . "./code.cg2js/cg-grid-widget.fasl")
(#xd839626a9a99ab62d29a0907a4a63498 . "./code.cg2js/cg-group-box.fasl")
(#x9e698e741641fb7dfc60a2b4966d61f6 . "./code.cg2js/cg-header-control.fasl")
(#x93d4be1b3f498001ae830b5342a1c3a2 . "./code.cg2js/cg-hotspot.fasl")
(#xa70942ebcfdfd7999be5cdfc1b36ee43 . "./code.cg2js/cg-html-dialog.fasl")
(#x95a8e05f091e8d4dd1ad581cd45f9025 . "./code.cg2js/cg-html-widget.fasl")
(#x1174800c8a5b6ce9630dc3c584791bc7 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x7ab9bc4d1cab17df8de0536f25f06bd2 . "./code.cg2js/cg-icon.fasl")
(#xb90dc3806fd9cedb26196c4074e52290 . "./code.cg2js/cg-ie.fasl")
(#xd65a577dc185967b7cd3944fa78a8eba . "./code.cg2js/cg-intersect-posbox.fasl")
(#x41aa594887c0d361f15f2444b2eefb5b . "./code.cg2js/cg-intersect.fasl")
(#xb92e4dd68bb9d88155f2448a82fee69e . "./code.cg2js/cg-item-list.fasl")
(#x684a36b570ae871d1988e5e8894b6629 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x3145989814e38fc216877d2f1a684bf4 . "./code.cg2js/cg-lamp.fasl")
(#xca6eef1b5cc9cc3944af9707d0120e0c . "./code.cg2js/cg-layout.fasl")
(#xdf04f310952e694595fcad4dfd339ef0 . "./code.cg2js/cg-lettered-menu.fasl")
(#x5833f8107c2a904653cb90045d61f405 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xa3f14f6f96068993e9fda15dcd234f55 . "./code.cg2js/cg-lisp-text.fasl")
(#x5b54cc3c1c4bf93d4fa777253d3bd115 . "./code.cg2js/cg-lisp-widget.fasl")
(#x529aec649c3fdc109f84994c795ebb37 . "./code.cg2js/cg-list-view.fasl")
(#xf2b71e759dff98ed0ae20d3338911176 . "./code.cg2js/cg-mci.fasl")
(#xa4822a23eff3c078ec717a025d0cf728 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xdbb21f9f6f8ad6153b5cb729a2e0fbe6 . "./code.cg2js/cg-menu.fasl")
(#xa28d9699c8e2b2d7724882ed50ca1711 . "./code.cg2js/cg-message-dialog.fasl")
(#x2f621286ade605af2d2688e1bad7d4e0 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xfbc243a53d9038fe211cf79ce68ce1e2 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xf51ce1ec3dcd4a8e22816777ba27e726 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x44a5f9b8743416c72843b17f991f57d9 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x7381e12e0b41688b4a1fcfb1d38b2546 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xdbd3ef32809753e934f0e03ed62079db . "./code.cg2js/cg-nodes.fasl")
(#x0719f5ed32509776ad9f94e2ff8675ae . "./code.cg2js/cg-object-editor-layout.fasl")
(#x420d3c81ede3d48bf55c506acce06b02 . "./code.cg2js/cg-object-editor.fasl")
(#xdc5945ac1004d9c126cdbb73825d7621 . "./code.cg2js/cg-ocx.fasl")
(#x926d93a66cac623e396d2b614cfb617a . "./code.cg2js/cg-os-widget.fasl")
(#x94a0c5a208fe138fca0a9313a4247289 . "./code.cg2js/cg-os-window.fasl")
(#x2973b8d6bee3d17ebb47e47fc33bec90 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x8049e5ec8ecba7a7f44b21147031ee93 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x8d6c1b4cea5c3d865acf6c52b9d11361 . "./code.cg2js/cg-outline.fasl")
(#x2825fa03f437ed3dfe15c06a86a19b56 . "./code.cg2js/cg-palette.fasl")
(#xab46bc51ade287ca91d142dd861744f4 . "./code.cg2js/cg-paren-matching.fasl")
(#x1381a665a9b7644a3f0aba69467107b4 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x14ebefb20c8f129392a92aef864d66fe . "./code.cg2js/cg-picture-widget.fasl")
(#x689c40a5666147d7860c67ca16fdd694 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x3b03a4ce3bb4fa0c9f895e9b85836972 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xf2f60e22771a17fc66d5f34b99da85ae . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xa71f06479d08c7af6f012ba0504bf7e5 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x259aa1104d2551c4b918df2684027dd5 . "./code.cg2js/cg-pixmap.fasl")
(#xafc3e63cd5579eccf41cafc2a654578f . "./code.cg2js/cg-plot-widget.fasl")
(#x1bb87c1f0d14058a15c6b8a8dd1a51e3 . "./code.cg2js/cg-printing.fasl")
(#x68e304dde09f93fe5ca48e2ce8e1af60 . "./code.cg2js/cg-progress-indicator.fasl")
(#x2a3d7604c482373c50a3e43806eea73e . "./code.cg2js/cg-project-window.fasl")
(#x1c008af9f352bcc05f00f0ccfac9353c . "./code.cg2js/cg-property.fasl")
(#xcec3bde4e6a32eb090a321bb55ac129f . "./code.cg2js/cg-radio-button.fasl")
(#x798e55065cce4bc2ad02e015152bf80c . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xcde0b5a46c6b6e8e54e02d795758fa70 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x315a74c1a477a64e79f1c5dd48487b32 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x28b9de1ee974c2fc70459507df45c451 . "./code.cg2js/cg-rich-edit.fasl")
(#x2748566a70ae21fec800700a443d24f8 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x7d4bbad5d37d2b249e9a62f06757201e . "./code.cg2js/cg-scaling-stream.fasl")
(#xe228502c4c9b3f048cae25e3d5473f4f . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x3db69e028962e9373858e7223193f595 . "./code.cg2js/cg-scroll-bar.fasl")
(#x7967350f4b99645430b0836531a2fce1 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xa027d1ce4e249cf0a7daafaeacb53766 . "./code.cg2js/cg-selected-object.fasl")
(#xbad6af0d540859aa60516ca9e910306d . "./code.cg2js/cg-shortcut-menu.fasl")
(#xe8bd4d89ba87274c83625f7c11c906c9 . "./code.cg2js/cg-split-bar.fasl")
(#x156bc8592cedd14b6d16f042a3834da2 . "./code.cg2js/cg-static-text.fasl")
(#x1e1f8be74f1d558ef0cebb694473ff9a . "./code.cg2js/cg-status-bar.fasl")
(#xce79c0260d37f0e88d76893caf946da5 . "./code.cg2js/cg-string-dialog.fasl")
(#xba32d030b45450e032eee6a2bd6332b6 . "./code.cg2js/cg-tab-control.fasl")
(#xca78a2bbe49e46717153851496274df6 . "./code.cg2js/cg-template-string.fasl")
(#xd971e4ffa9fa779707746fb7c41f9464 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xbfe7736899b0b7e5ad21d6c05ec42eb2 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xc8d8ea0d43bda6e32ca4080c7f00beaa . "./code.cg2js/cg-text-edit-pane.fasl")
(#xb58ab43d7a80e39239872d5217184e72 . "./code.cg2js/cg-text-or-combo.fasl")
(#x78c905fcadf72aef7efa99518075cfc2 . "./code.cg2js/cg-text-widget.fasl")
(#x01dd0c726fe0dbd27f09ae2acbe0479e . "./code.cg2js/cg-timer.fasl")
(#xa0043ee9e3b74274c30e3668a8edbed6 . "./code.cg2js/cg-toggling-widget.fasl")
(#x515206e7b513e055b8c4e89560dc8aef . "./code.cg2js/cg-toolbar.fasl")
(#xc062b8e80cf70ee6bcd752c6267ea95a . "./code.cg2js/cg-tooltip.fasl")
(#x963cd261c77ccbdd46476040ae153614 . "./code.cg2js/cg-touch.fasl")
(#xc2a8ad6555ad214410187a6237602521 . "./code.cg2js/cg-trackbar.fasl")
(#x7d16b9bf2fa72f590e40c6d0aafb4ffc . "./code.cg2js/cg-tray.fasl")
(#xaa839f9f6fbfbca4252c27af56a4bb68 . "./code.cg2js/cg-up-down-control.fasl")
(#xfc665ece748bba5d90436145e3fe96d3 . "./code.cg2js/cg-utility-dialog.fasl")
(#x3a8eb6571d4eb4add0024715d2f38ac3 . "./code.cg2js/cg-web-browser-dde.fasl")
(#xe443066cc9816c2cf974d7a728290a4a . "./code.cg2js/cg-web-browser.fasl")
(#x52ccfea5a81102a7c5a289d9e767d368 . "./code.cg2js/cg-wrap-string.fasl")
(#x2d158a6d2b6377ad6129cd9e5634c9a7 . "./code.cg2js/cg-yes-no-list.fasl")
(#x6941667cd8520954b2ffb782d2c95c65 . "./code.cg2js/cg-yes-no-string.fasl")
(#x59223f2edecc2b96ac65491c1f6842cf . "./code.cg2js/cg.fasl")
(#xa7427d58e6c3ef5800b9cd98147ebbbb . "./code.idejs/ide-acache.fasl")
(#x7f259ec1c4edae2c471056293eb20f05 . "./code.idejs/ide-apropos.fasl")
(#xbe16c9129b7ea34dd3b672cca57c68b6 . "./code.idejs/ide-base.fasl")
(#x2d2f0e1749de7a802c911a22265b4369 . "./code.idejs/ide-class-browser.fasl")
(#x76c9a8b822d822460678f636c3fa21cd . "./code.idejs/ide-class-grapher.fasl")
(#x158648066593c83cddecb0385c12f20a . "./code.idejs/ide-clipboard.fasl")
(#x088876c88708187c960269a9437401df . "./code.idejs/ide-dev.fasl")
(#x6a716bb658dc7bf46eea6aa8fbbe0299 . "./code.idejs/ide-editor.fasl")
(#xa3e2bbf08d33a30f737335d00592d7d5 . "./code.idejs/ide-find-definitions.fasl")
(#xb3ba76ff69b021c4eec4f6f8103d6b72 . "./code.idejs/ide-find-in-files.fasl")
(#x8ef4ab123a9a10aa7aef3bcfb8af0d83 . "./code.idejs/ide-form.fasl")
(#xa76dc6f8481e4092ea12094d233d4a5a . "./code.idejs/ide-grapher.fasl")
(#xef99b73aed709df853b8729a5a720f69 . "./code.idejs/ide-inspector.fasl")
(#x380d73fd2e16be6f7beeb354ef2cf0c4 . "./code.idejs/ide-intro.fasl")
(#x765ccb9fb48ec90695392df777363b86 . "./code.idejs/ide-keyshort.fasl")
(#x006183b828e8a94e1addf392ea17a622 . "./code.idejs/ide-navigator.fasl")
(#x24581c8c7a26fa45bfcae1e8a61407b6 . "./code.idejs/ide-package-list.fasl")
(#x3dbfb4bc3c60a7ac4442e2d6f35ea44d . "./code.idejs/ide-patch-dialog.fasl")
(#x4b2e08cbf74546e2fcee41644646e649 . "./code.idejs/ide-process-browser.fasl")
(#x5e4ff2ccbf375c9979fe1a9f78f29b4c . "./code.idejs/ide-project.fasl")
(#x13472c6cb1fcec49221900b56554dae1 . "./code.idejs/ide-trace.fasl")
(#x7c3dd95758248883b5e954e3047e2499 . "./code.idejs/ide-window-list.fasl")
(#x6104f0792ca45b46cb4554581fe35224 . "./code.idejs/ide.fasl")
(#x82407180256cdbcae17acb711a3ed09f . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
